package edu.sfedu.brsback.controllers.employee;

import edu.sfedu.brsback.dto.base.ListResponse;
import edu.sfedu.brsback.dto.module.ModuleFilter;
import edu.sfedu.brsback.dto.module.ModuleResponse;
import edu.sfedu.brsback.dto.module.ModuleShortResponse;
import edu.sfedu.brsback.entities.Module;
import edu.sfedu.brsback.services.ModuleService;
import edu.sfedu.brsback.utils.auth.EmployeeAuth;
import edu.sfedu.brsback.utils.auth.StudentAuth;
import jakarta.annotation.Nullable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@RestController
@RequestMapping("/employee/module")
@RequiredArgsConstructor
@EmployeeAuth
public class ModuleController {
    private final ModuleService moduleService;

    @GetMapping("")
    public ResponseEntity<?> getModules(
            @RequestBody ModuleFilter moduleFilter
    ){
        var modules = moduleService.getAvaliableModules(moduleFilter);
        return ResponseEntity.ok(
                new ListResponse<>(
                        modules.stream()
                                .map(ModuleShortResponse::new)
                                .toList()
                )
        );
    }

    @GetMapping("/{moduleId}")
    public ResponseEntity<?> getModuleById(
            @PathVariable Long moduleId
    ){
        var module = moduleService.getModuleById(moduleId);
        return ResponseEntity.ok(new ModuleResponse(module));
    }
}
