package edu.sfedu.brsback.controllers.student;

import edu.sfedu.brsback.dto.base.ListResponse;
import edu.sfedu.brsback.dto.module.ModuleFilter;
import edu.sfedu.brsback.dto.module.ModuleResponse;
import edu.sfedu.brsback.dto.module.ModuleShortResponse;
import edu.sfedu.brsback.entities.Module;
import edu.sfedu.brsback.services.ModuleService;
import edu.sfedu.brsback.utils.auth.StudentAuth;
import jakarta.annotation.Nullable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@RestController
@RequestMapping("/student/module")
@RequiredArgsConstructor
@StudentAuth
public class ModuleController {
    private final ModuleService moduleService;

    @GetMapping("")
    public ResponseEntity<?> getModules(
            @RequestBody ModuleFilter moduleFilter
    ){
        var modules = moduleService.getAvaliableModules(moduleFilter);
        Long recordBookId = moduleFilter.getRecordBookId();
        if (recordBookId == null){
            return ResponseEntity.badRequest().body(Map.of("error", "ERROR.NO_RECORD_BOOK_ID"));
        }
        return ResponseEntity.ok(
                new ListResponse<>(
                        modules.stream()
                                .map((Module module) -> new ModuleShortResponse(
                                        module,
                                        recordBookId
                                ))
                                .toList()
                )
        );
    }
    @GetMapping("/{moduleId}")
    public ResponseEntity<?> getModuleById(
            @PathVariable Long moduleId,
            @RequestParam("record_book_id") Long recordBookId
    ){
        var module = moduleService.getModuleById(moduleId);
        return ResponseEntity.ok(new ModuleResponse(module, recordBookId));
    }
}
