package edu.sfedu.brsback.services.implementations;

import edu.sfedu.brsback.dto.score.ScoreSetDto;
import edu.sfedu.brsback.entities.Module;
import edu.sfedu.brsback.entities.RecordBook;
import edu.sfedu.brsback.entities.Score;
import edu.sfedu.brsback.exceptions.AccessDeniedException;
import edu.sfedu.brsback.exceptions.ModelNotFoundException;
import edu.sfedu.brsback.exceptions.WrongScoreException;
import edu.sfedu.brsback.repos.RecordBookRepository;
import edu.sfedu.brsback.repos.ScoreRepository;
import edu.sfedu.brsback.services.ModuleRightService;
import edu.sfedu.brsback.services.ModuleService;
import edu.sfedu.brsback.services.ScoreService;
import edu.sfedu.brsback.services.SecurityService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class ScoreServiceImpl implements ScoreService {
    private final ScoreRepository scoreRepository;
    private final RecordBookRepository recordBookRepository;
    private final ModuleService moduleService;
    private final SecurityService securityService;
    private final ModuleRightService moduleRightService;

    @Override
    public void setScore(int value, Module module, RecordBook recordBook) {
        checkAccess(module);

        if (module.getMaxScore() < value){
            throw new WrongScoreException();
        }

        Score score = new Score(value, recordBook, module);
        scoreRepository.save(score);
    }

    @Override
    public void setScore(ScoreSetDto scoreSetDto) {
        var module = moduleService.getModuleById(scoreSetDto.getModuleId());
        checkAccess(module);

        if (module.getMaxScore() < scoreSetDto.getScore()){
            throw new WrongScoreException();
        }

        Score score = new Score(
                scoreSetDto.getScore(),
                recordBookRepository.findById(scoreSetDto.getRecordBookId()).orElseThrow(ModelNotFoundException::new),
                module
        );
        scoreRepository.save(score);
    }

    private void checkAccess(Module module) {
        var right = moduleRightService.getRightOnModule(
                module,
                securityService.getAuthUser()
        );

        if (right.isRScore()){
            throw new AccessDeniedException();
        }
    }
}
